# frozen_string_literal: true

module Hermes
  module Api
    module Dmo
      # checker Api::Dmo::HermesController avant d'ajouter des methodes
      class HousingOffersController < Hermes::Api::Dmo::ApplicationController
        hermes_tools 'aggregations'
        require Rails.root.join('lib', 'core_extentions', 'parameters', 'handle_sort').to_s
        set_instance_for :eligible_requests, :with_validated_applications_for, :history

        before_action :sanitize_params, only: %i[create update]

        VALIDATED_APPLICATION_STATUS = 'TRANSMIS BAILLEUR'

        MANAGEMENT_ROLES = %w[
          Administrator
          Manager
          entreprise
          bailleur
        ].freeze

        VALID_OPERATORS = %w[
          $in
          $nin
          $ne
          $gt
          $gte
          $lt
          $lte
          $regex
        ].freeze

        ACCEPTABLE_PARAMS = %w[
          requalification
          dalo
          ids
          als_id
          entry_date
          created_at
          updated_at
          external_ref
          communicate_exact_address
          eligibility_query
          typology
          lessor_status
          type
          age
          district
          department
          publication_end_date
          rent_amount
          rent_with_charges
          surface
          rooms
          exclusivity_end_date
          nb_of_eligible_employees
          nb_of_requests_from_employees
          availability_date
          description
          details
          old_status
          status_updated_at
          effort_rate
          min_effort_treshold
          single_parent
          reserved
          priority_public
          mobility
          bedrooms
          bathrooms
          heating_type
          hot_water_distribution
          floors
          elevators
          has_elevator
          has_gardian
          outside
          cellars
          has_cellar
          parking_charges
          parking_type
          pmr
          kitchen_type
          mutation
          threshold_category_1
          threshold_type
          threshold_subtype
          threshold_category_2
          threshold_category_3
          threshold_category_4
          threshold_category_5
          threshold_category_6
          threshold_category_7
          threshold_category_8
          threshold_category_9
          threshold_category_10
          threshold_category_11
          threshold_category_12
          individual_heating
          date_publication_start
          date_publication_end
          lessor_id
          kind
          building_type
          already_rent
          guarantee_deposit
          rental_charges
          offer_status
          offer_status_updated_at
          lessor_status_updated_at
          ceil_category_1
          ceil_category_2
          ceil_category_3
          ceil_category_4
          ceil_category_5
          ceil_category_6
          ceil_category_7
          ceil_category_8
          ceil_category_9
          ceil_category_10
          ceil_category_11
          ceil_category_12
          floor_category_1
          floor_category_2
          floor_category_3
          floor_category_4
          floor_category_5
          floor_category_6
          floor_category_7
          floor_category_8
          floor_category_9
          floor_category_10
          floor_category_11
          reserving_siren
          floor_category_12
          min_revenues
          total_charges
          max_effort_rate
          charges
          tense_area
          subsidiary_company
          rpls_id
          operation_name
          administrator
          administration_area
          potential_request_name_town
          potential_request_name_county
          number_of_companies_30_minutes_around
          construction_type
          construction_year
          address_can_be_shared
          qpv
          total_floors
          address_extended
          test_created_at
          already_inhabited
          new_offer
          lessor_name
          lessor_siren
          als_siren
          postal_code
          insee_code
          postal_codes
          residence_code
          residence_title
          pictures
          agency_id
          agency_name
          lessor_contact_name
          lessor_email
          lessor_phone_number
          gardens
          has_garden
          green_area
          condo
          furnished_lodging
          virtual_visit
          duplex
          first_quartile
          other_quartiles
          lessor_convention_reference
          reserving_agency_name
          reserving_agency_siren
          disability_accesses
          has_disability_access
          terraces_or_balconies
          first_quartile
          reserving_business_name
          address_complements
          is_mad
          is_mad_updated_at
          mad_als_date
          reservation_agreement_ref
          designation_date_end
          lessor_orientation
          dpe_ges
          dpe_conso
          entities_siret
          address
          postal_code
          country
          latitude
          longitude
          target_objective_ids
          auto_destroy_next_day
          last_update_by_lessor
        ].freeze

        REGEX_PARAMS = %w[
        ].freeze

        HOUSING_REQUESTS_ACCEPTABLE_PARAMS = %w[
          is_active
          als_id
          request_status
          attributed_housing_district
          attributed_housing_district_postal_code
          old_request_status
          request_status_updated_at
          old_entity_status
          updated_entity_status
          entity_status
          dalo
          asked_mobility
          valid_mobility
          decider
          submitted_to_cal_nb
          sent_to_cal_nb
          rejected_in_cal_nb
          rejected_after_cal_nb
          accepted_in_cal_nb
          mobility
          seeked_types
          entity_help_required
          contract_type
          registration_number
          applicant_ids
          entity_validated
          priority_points_indexes
          nud
          nb_of_requests_made
          reference_tax_revenues
          seeked_districts
          category_type
          single_parent
          id_counter
          id_unique
          id_unique_associate
          internal_ref
          forcing_duplicate
          electronical_renewal
          ec_law_ressources
          is_long_delay_anormal
          date_calculator_indicator
          with_car_park
          widen_request
          housing_type_list
          expulsion_judgement_date
          other_reason
          work_contract_type
          seeked_district_postal_codes
          seeked_departments
          seeked_localisation_list
          dependents
          monthly_revenues
          typology_range
          entity_validated
          family_size
          coapplicants
          lessor_name
          lessor_siren
          social_housing_entry_date
          social_housing_name
          test_created_at
          sne_number
          employee_id
          employee_start_date
          entity_access_level
          employee_unit
          tab_state
          motivation1
          motivation2
          motivation3
          seeking_first_floor
          seeking_elevator
          seeked_max_rent
          sne_creation_date
          precisions
          has_multiple_attributions_under_conditions
          seeked_housing_categories
          min_surface
          main_applicant_id
          requester_firstname
          requester_lastname
          requester_fullname
          requester_email
          entity_id
          coapplicant_ids
          coapplicants
          seeked_epci_first_quartils
          seeked_quartil_belongings
          dependents_monthly_incomes
          dependents_monthly_charges
          housing_offer_ids
          radiation_date
          radiation_motive
          commercialized
          is_public
          last_update_by_lessor
        ].freeze

        HOUSING_REQUESTS_REGEX_PARAMS = %w[].freeze

        HOUSING_APPLICATIONS_ACCEPTABLE_PARAMS = %w[
          housing_request_id
          housing_offer_id
          epci
          max_date
          applicated_at
          effort_rate
          status
          status_updated_at
        ].freeze

        HOUSING_APPLICATIONS_REGEX_PARAMS = %w[].freeze

        MUTATE_PARAMS = %i[
          requalification
          dalo
          entry_date
          external_ref
          communicate_exact_address
          typology
          lessor_status
          type
          age
          district
          department
          publication_end_date
          rent_amount
          surface
          rooms
          exclusivity_end_date
          nb_of_eligible_employees
          nb_of_requests_from_employees
          availability_date
          description
          details
          effort_rate
          min_effort_treshold
          reserved
          priority_public
          mobility
          bedrooms
          bathrooms
          heating_type
          hot_water_distribution
          floor
          floors
          elevators
          has_gardian
          outside
          cellars
          parking_charges
          parking_type
          reserving_siren
          address_complements
          pmr
          kitchen_type
          mutation
          threshold_type
          threshold_subtype
          individual_heating
          date_publication_start
          date_publication_end
          lessor_id
          kind
          building_type
          allready_rent
          guarantee_deposit
          rental_charges
          offer_status
          min_effort_rate
          already_rent
          tense_area
          subsidiary_company
          rpls_id
          operation_name
          administrator
          rent_with_charges
          reserving_business_name
          administration_area
          potential_request_name_town
          potential_request_name_county
          number_of_companies_30_minutes_around
          construction_type
          construction_year
          address_can_be_shared
          qpv
          total_floors
          address_extended
          test_created_at
          already_inhabited
          new_offer
          als_siren
          postal_code
          insee_code
          residence_code
          residence_title
          agency_id
          agency_name
          lessor_contact_name
          lessor_email
          lessor_phone_number
          gardens
          green_area
          condo
          furnished_lodging
          virtual_visit
          duplex
          first_quartile
          other_quartiles
          lessor_convention_reference
          reserving_agency_name
          reserving_agency_siren
          disability_accesses
          terraces_or_balconies
          first_quartile
          mad_als_date
          reservation_agreement_ref
          designation_date_end
          lessor_orientation
          dpe_ges
          dpe_conso
          ordered_picture_ids
          entities_siret
          commercialized
          is_public
          single_parent
          address
          postal_code
          country
          latitude
          longitude
          auto_destroy_next_day
          last_update_by_lessor
        ].freeze

        LESSOR_FORBIDDEN_PARAMS = %i[
          als_siren
          dalo
          status
          status_updated_at
        ]

        mutate_params_only *MUTATE_PARAMS

        def handle_offers_eligibility
          applicant = Hermes::Applicant.find_by(id: @user&.main_applicant_id)
          @query_precision ||= {}
          @query_precision.merge!(
            if %w[Administrator bailleur].include? current_user_roles
              {}
            elsif applicant&.eligibility_query
              applicant.eligibility_query.json_to_h.merge(
                offer_status: {:$in => HousingOffer.published_statuses}
              )
            else
              {public: true}
            end
          )
        end

        def include_relationships
          [:scores, :pictures]
        end

        def index_parameters_settings
          acceptable_params *ACCEPTABLE_PARAMS
          regex_params *REGEX_PARAMS
          valid_operators *VALID_OPERATORS
        end

        def create
          @model.current_user_id = @user.id
          user_id = @user.id
          param_user = params&.fetch(:data, nil)&.fetch(:attributes, nil)&.fetch(:user_id, nil)
          Rails.logger.debug "TEST before #{user_id}"
          user_id = param_user if @user.has_role?('Administrator') && param_user
          Rails.logger.info "TEST #{mutate_valid_params} after #{user_id}"
          sanitize_status
          @instance = @model.new(mutate_valid_params.merge(user_id: user_id))

          if (found_instance = Hermes::HousingOffer.find_by(external_ref: @instance.external_ref, lessor_siren: @instance.lessor_siren))

            @instance = found_instance
            @instance.with_lock(reload: true) do
              return render_json_error(:unprocessable_entity, :not_writers) unless @instance.reload && @instance.writable_by?(@user)
              valid_params = mutate_valid_params
              valid_params[:user_id] = params['data']['attributes'][:user_id] if @user.has_role?('Administrator')
              return render_json_error(:parameters_not_present, :no_valid_parameters) unless valid_params.present?
              Rails.logger.info "TEST valid_params are #{valid_params}"
              @instance.last_update_by_lessor =  DateTime.current
              return respond_with_errors(@instance) unless @instance.update(valid_params) && @instance.save

              render(jsonapi: @instance, serializer: @serializer)
            end
          else
            @instance.with_lock(reload: true) do
              @instance.last_update_by_lessor =  DateTime.current
              return respond_with_errors(@instance) unless @instance.save
              render(jsonapi: @instance, serializer: @serializer, status: :created)
            end
          end
        end

        def index
          index_parameters_settings

          aggregation = if %w[Administrator bailleur entreprise superadministrateur].any? {|role| current_user_roles.include?(role)}
                          index_aggregation
                        elsif is_collaborator_offer
                          index_aggregation
                        elsif current_user_roles.include?('reservataire') && @user.reservation_holder_id
                          index_aggregation(no_acl: true).where(reservation_holder_id: @user.reservation_holder_id)
                        else
                          handle_offers_eligibility
                          index_aggregation(no_acl: true)
                        end
          roles_cond = %w[Administrator Manager].include?(@user&.roles) # || %w[].include?(@user&.auth_roles)
          scores = nil

          applicant = Hermes::Applicant.find_by(id: @user&.main_applicant_id)

          per_page = params[:per_page]&.to_i || 15
          page = params[:page]&.to_i || 1
          options = {}
          unless %w[bailleur entreprise].any? {|role| current_user_roles.include?(role)}
            scoring_params = params.slice(*SCORING_PARAMS)
            scoring_sorting_params = params[:sort]&.slice(*SCORING_PARAMS)
            if (scoring_params.present? || scoring_sorting_params.present?) && applicant
              aggregation = aggregate_with_score('offers', applicant.id, params, aggregation)
            end
            scores = if aggregation.is_a?(Mongo::Collection::View::Aggregation)
                       options[:entries_identifier] = 'foreignItem._id'
                       Hermes::Score.where(:id.in => aggregation.pluck(:id))
                     else
                       limited_aggregation = aggregation.only(:id).offset((page - 1) * per_page).limit(per_page)
                       Hermes::Score.where(:housing_offer_id.in => limited_aggregation.pluck(:id), applicant: applicant)
                     end
            options.merge!(scores: scores)
          end

          if params[:postal_codes] && params[:postal_codes].is_a?(Array)
            aggregation = aggregation.search_by_postal_codes(params[:postal_codes])
          end

          #Rails.logger.info "TEST #{aggregation.to_a}"
          paginate(
            aggregation,
            each_serializer: @serializer,
            **options
          )
        end

        def show
          @options = {
            include: %i[user hermes-lessor entities threshold],
            fields: {'hermes-lessor'.to_sym => ['title']}
          }
          base_query = if %w[Administrator bailleur entreprise superadministrateur].any? {|role| current_user_roles.include?(role)}
                         @model.readable_by(@user)
                       elsif current_user_roles.include?('reservataire')
                         @model
                       elsif is_collaborator_offer
                         @model
                       else
                         handle_offers_eligibility if @user
                         @model
                       end
          @instance = base_query.find_by(id: params[:id], **@query_precision) if params[:id]
          return render_json_error(:not_found, :object_not_found) if reservataire_not_matching_id && !is_collaborator_offer
          super
        end

        def update
          @model.current_user_id = @user.id
          sanitize_status
          if current_user_roles.include?('reservataire')
            @instance = @model.find(params[:id])
            return render_json_error(:not_found, :object_not_found) unless @instance
            return reservataire_not_matching_id ? render_json_error(:not_found, :object_not_found) : check_params_and_update
          end
          @instance.with_lock(reload: true) do
            super
          end

        end

        def destroy
          if current_user_roles.include?('reservataire')
            @instance = @model.find(params[:id])
            return render_json_error(:not_found, :object_not_found) unless @instance
            if reservataire_not_matching_id
              return render_json_error(:not_found, :object_not_found)
            else
              return render(json: @instance.errors, status: :unprocessable_entity) unless @instance.destroy
              return render(body: nil, status: :no_content)
            end
          end
          super
        end

        def is_collaborator_offer
          current_user.auth_roles&.include?('collaborateur offre logement')
        end
        
        def reservataire_not_matching_id
          (current_user_roles.include?('reservataire') && @user.reservation_holder_id != @instance.reservation_holder_id ) 
        end

        def all_eligibile_requests
          return Hermes::HousingRequest.where(id: "") unless @instance.eligibility_query

          eligibility = JSON.parse(@instance.eligibility_query).each_with_object({}) do |(key, val), h|
            h[key.to_sym] = val
          end
          filter = build_query(Hermes::HousingRequest).deep_merge(eligibility.merge(is_active: true))
          Hermes::HousingRequest.where(filter)
        end

        def eligible_requests
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:unprocessable_action, :not_manager) unless can_manage_offers?

          acceptable_params *HOUSING_REQUESTS_ACCEPTABLE_PARAMS
          regex_params *HOUSING_REQUESTS_REGEX_PARAMS
          valid_operators *VALID_OPERATORS

          for_offer = @instance
          aggregation = all_eligibile_requests

          per_page = params[:per_page]&.to_i || 15
          page = params[:page]&.to_i || 1
          options = {}
          scoring_params = params.slice(*SCORING_PARAMS)
          scoring_sorting_params = params[:sort]&.slice(*SCORING_PARAMS)
          if scoring_params.present? || scoring_sorting_params.present?
            aggregation = aggregate_with_score('requests', for_offer.id, params, aggregation)
          end
          scores = if aggregation&.is_a?(Mongo::Collection::View::Aggregation)
                     options[:entries_identifier] = 'foreignItem._id'
                     options[:entries_class] = Hermes::HousingRequest
                     Hermes::Score.where(:id.in => aggregation&.pluck(:id))
                   else
                     limited_aggregation = aggregation&.only(:id)&.offset((page - 1) * per_page)&.limit(per_page)
                     Hermes::Score.where(:applicant_id.in => limited_aggregation&.pluck(:main_applicant_id), housing_offer: for_offer)
                   end
          options.merge!(for_offer: for_offer, scores: scores)
          paginate(
            aggregation,
            each_serializer: Hermes::HousingRequestSerializer,
            **options
          )
        end

        def all_applications
          return render_json_error(:not_found, :instance_not_found) unless @instance
          # return render_json_error(:unprocessable_action, :not_manager) unless can_manage_offers?

          acceptable_params *HOUSING_APPLICATIONS_ACCEPTABLE_PARAMS
          regex_params *HOUSING_APPLICATIONS_REGEX_PARAMS
          valid_operators *VALID_OPERATORS

          per_page = params[:per_page]&.to_i || 15
          page = params[:page]&.to_i || 1
          options = {}
          housing_applications = Hermes::HousingApplication.readable_by(@user).where(housing_offer_id: @instance.id)
                               .order(order_params)
                               .includes(housing_request: [:main_applicant])
          options[:join_housing_request] = true
          paginate(
            housing_applications,
            each_serializer: Hermes::HousingApplicationSerializer,
            **options
          )
        end

        def paginate_applications(applications)
          acceptable_params *HOUSING_APPLICATIONS_ACCEPTABLE_PARAMS
          regex_params *HOUSING_APPLICATIONS_REGEX_PARAMS
          valid_operators *VALID_OPERATORS
          applications = applications.readable_by(@user).where(build_query(Hermes::HousingApplication))
                           .order(order_params)
                           .includes(housing_request: [:main_applicant])
          options = {}
          options[:join_housing_request] = true
          paginate(
            applications,
            each_serializer: HousingApplicationSerializer,
            **options
          )
        end

        def selected_applications
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:unprocessable_action, :not_manager) unless can_manage_offers? #todo : a vérifier lorsqu'on aura la gestion de droits spécifiée

          applications = Hermes::HousingApplication.selected_by_offer @instance.id
          paginate_applications(applications)
        end

        def can_manage_offers?
          (MANAGEMENT_ROLES & current_user_roles)&.length&.send('>', 0)
        end

        def post_selected_applications
          return render_json_error(:not_found, :instance_not_found) unless @instance
          return render_json_error(:unprocessable_action, :not_manager) unless can_manage_offers? #todo : a vérifier lorsqu'on aura la gestion de droits spécifiée

          als_ids = params['data']['attributes'][:als_request_ids] if params['data'] && params['data']['attributes']
          return render_json_error(:parameters_not_present, :no_valid_parameters) unless als_ids.present?

          concerned_housing_requests_ids = Hermes::HousingRequest.where(:als_id => {'$in' => als_ids}).pluck(:id)
          concerned_housing_applications = Hermes::HousingApplication.where(:housing_request_id => {'$in' => concerned_housing_requests_ids}, housing_offer_id: @instance.id)
          has_error = false
          concerned_housing_applications.each do |ha|
            ha.status = "TRANSMIS BAILLEUR"
            has_error = true unless ha.save
          end
          options = {}
          options[:join_housing_request] = true
          paginate(
            concerned_housing_applications,
            each_serializer: HousingApplicationSerializer,
            join_housing_request: true
          ) unless has_error
        end

        def from_param
          from = params['from']
          return render_json_error(:unprocessable_entity, :missing_from_parameter) unless from
          begin
            DateTime.parse(from)
          rescue
            return render_json_error(:unprocessable_entity, :malformed_from_parameter)
          end
        end

        def from_to_query
          from = from_param
          return from unless from.is_a?(DateTime)

          query = {status_updated_at: {:$gte => from}}
          to = DateTime.parse(params[:to]) rescue nil
          query[:status_updated_at][:$lte] = to if to.is_a? DateTime
          query
        end

        def with_validated_applications
          from_to = from_to_query
          return from_to unless from_to.is_a?(Hash)

          application_query = {
            status: VALIDATED_APPLICATION_STATUS
          }.merge(from_to)

          validated_applications = Hermes::HousingApplication.where(
            application_query
          )
          housing_offer_ids = validated_applications.pluck(:housing_offer_id)
                                .uniq
                                .compact
          @query_precision = {:id.in => housing_offer_ids}
          index_parameters_settings
          aggregation = index_aggregation
          options = {}

          unless ['false', false].include?(params[:pagination])
            paginate(
              aggregation,
              each_serializer: Hermes::HousingOfferSerializer,
              **options
            )
          else
            render(
              jsonapi: aggregation,
              each_serializer: Hermes::HousingOfferSerializer,
              **options
            )
          end
        end

        def with_validated_applications_for
          def include_relationships
            []
          end

          from_to = from_to_query
          return from_to unless from_to.is_a?(Hash)
          return render_json_error(:not_found, :object_not_found) unless @instance

          acceptable_params *HOUSING_APPLICATIONS_ACCEPTABLE_PARAMS
          regex_params *HOUSING_APPLICATIONS_REGEX_PARAMS
          valid_operators *VALID_OPERATORS

          applications_query = {
            housing_offer_id: @instance.id,
            status: VALIDATED_APPLICATION_STATUS
          }.merge(from_to)
          @query_precision = applications_query
          aggregation = index_aggregation(target_model: Hermes::HousingApplication)
          options = {embedded_applicant: true}

          unless ['false', false].include?(params[:pagination])
            paginate(
              aggregation,
              each_serializer: Hermes::HousingApplicationSerializer,
              **options
            )
          else
            render(
              jsonapi: aggregation,
              each_serializer: Hermes::HousingApplicationSerializer,
              **options
            )
          end
        end

        def paginate_history(history)
          acceptable_params *%i[lessor_objective_id]
          history = history.where(build_query(Hermes::LessorObjectiveHistory)).order(order_params)
          paginate(
            history,
            each_serializer: Hermes::LessorObjectiveHistorySerializer
          )
        end

        def history
          if current_user_roles.include?('reservataire')
            @instance = @model.find_by(id: params[:id], **@query_precision) if params[:id]
            return render_json_error(:not_found, :object_not_found) if reservataire_not_matching_id
          end
          if current_user_roles.include?('bailleur')
            @instance = @model.readable_by(@user).find_by(id: params[:id], **@query_precision) if params[:id]
          end

          return render_json_error(:not_found, :instance_not_found) unless @instance

          paginate_history(@instance.history)
        end

        def paginate_detailed_score(detailed_score)
          acceptable_params *%i[applicant_id]
          detailed_score = detailed_score.where(build_query(Hermes::Score)).order(order_params)

          if current_user_roles.include?('demandeur')
            applicant = Hermes::Applicant.find_by(id: @user&.main_applicant_id)
            detailed_score = detailed_score.where(applicant: applicant)
          else
            applicant_ids = Hermes::Applicant.readable_by(@user).pluck(:id).map(&:to_s)
            params.require(:applicant_id) if applicant_ids.count.positive?

            return render_json_error(:forbidden, :no_readable_permission) unless applicant_ids.include? params[:applicant_id]
          end

          paginate(
            detailed_score,
            each_serializer: Hermes::ScoreSerializer
          )
        end

        def detailed_score
          @instance = @model.find_by(id: params[:id], **@query_precision) if params[:id]
          return render_json_error(:not_found, :instance_not_found) unless @instance

          paginate_detailed_score(@instance.detailed_score)

        rescue ActionController::ParameterMissing => e
          render_json_error(:not_acceptable, :parameter_missing, { message: e.message })
        end

        def score_position
          offer = Hermes::HousingOffer.find_by(id: params[:id]) if params[:id]

          if current_user_roles.include?('demandeur')
            applicant_id = @user&.main_applicant_id
          else
            params.require(:applicant_id)
            applicant_id = Hermes::Applicant.readable_by(@user).find_by(id: params[:applicant_id])&.id

            return render_json_error(:forbidden, :no_readable_permission) unless applicant_id
          end

          return render_json_error(:not_found, :instance_not_found) unless offer

          score = Hermes::Score.find_by(
            housing_offer_id: offer.id,
            applicant_id: applicant_id
          )

          response = if score
                      score.located_eligible_position
                    else
                      {
                        position_by_department: 'uneligible',
                        total_by_department: 'uneligible',
                        position_by_postal_code: 'uneligible',
                        total_by_postal_code: 'uneligible'
                      }
                    end
          render json: response, status: :ok

        rescue ActionController::ParameterMissing => e
          render_json_error(:not_acceptable, :parameter_missing, { message: e.message })
        end

        private

        def sanitize_params
          if current_lessor
            @@mutate_params_only = *(@@mutate_params_only - LESSOR_FORBIDDEN_PARAMS)
          else
            @@mutate_params_only = *(@@mutate_params_only + LESSOR_FORBIDDEN_PARAMS)
          end
        end

        def sanitize_status
          # only offer.lessor can setup Hermes::HousingOffer::ATTRIBUTION_STATUSES
          if Hermes::HousingOffer::ATTRIBUTION_STATUSES.include?(params[:data][:attributes][:offer_status])
            if current_lessor.nil? || current_lessor&.id&.to_s != mutate_valid_params[:lessor_id]
              @instance = @model.find(params[:id] || 0)
              params[:data][:attributes][:offer_status] = @instance&.offer_status_was
            end
          end
        end
      end
    end
  end
end
